@@ -72,4 +72,9 @@ sub checkNameSfa {
     return $name;
 }
 
+sub checkCsrf {
+    my ( $self, $req ) = @_;
+
+    return $req->headers->header('X-CSRF-Check');
+}
 1;
