@@ -56,6 +56,9 @@ sub run {
     # Verification that user has a valid password
     if ( $action eq 'verify' ) {
 
+        $self->checkCsrf($req)
+            or return $self->p->sendError( $req, 'csrfError', 400 );
+
         # Check Password
         my $password       = $req->param('password');
         my $passwordverify = $req->param('passwordverify');
@@ -129,6 +132,9 @@ sub run {
     }
     elsif ( $action eq 'delete' ) {
 
+        $self->checkCsrf($req)
+            or return $self->p->sendError( $req, 'csrfError', 400 );
+
         # Check if unregistration is allowed
         return $self->p->sendError( $req, 'notAuthorized', 400 )
           unless $self->conf->{password2fUserCanRemoveKey};
