@@ -41,6 +41,9 @@ sub run {
     # Verification that user has a valid TOTP app
     if ( $action eq 'verify' ) {
 
+        $self->checkCsrf($req)
+            or return $self->p->sendError( $req, 'csrfError', 400 );
+
         # Get form token
         my $token = $req->param('token');
         unless ($token) {
@@ -140,6 +143,9 @@ sub run {
     elsif ( $action eq 'getkey' ) {
         my ( $nk, $secret, $issuer ) = ( 0, '' );
 
+        $self->checkCsrf($req)
+            or return $self->p->sendError( $req, 'csrfError', 400 );
+
         # Read existing TOTP 2F
         my @totp2f =
           $self->find2fDevicesByType( $req, $req->userData, $self->type );
@@ -184,6 +190,9 @@ sub run {
     # Delete TOTP
     elsif ( $action eq 'delete' ) {
 
+        $self->checkCsrf($req)
+            or return $self->p->sendError( $req, 'csrfError', 400 );
+
         # Check if unregistration is allowed
         return $self->p->sendError( $req, 'notAuthorized', 400 )
           unless $self->userCanRemove;
