@@ -202,6 +202,9 @@ sub run {
 
     elsif ( $action eq 'delete' ) {
 
+        $self->checkCsrf($req)
+            or return $self->p->sendError( $req, 'csrfError', 400 );
+
         # Check if unregistration is allowed
         return $self->p->sendError( $req, 'notAuthorized', 400 )
           unless $self->userCanRemove;
