@@ -7,17 +7,17 @@ use JSON qw(from_json to_json);
 use MIME::Base64 qw(encode_base64url decode_base64url);
 use Crypt::URandom;
 
-our $VERSION = '2.0.15';
+our $VERSION = '2.0.16';
 
 extends 'Lemonldap::NG::Portal::2F::Register::Base';
 with 'Lemonldap::NG::Portal::Lib::WebAuthn';
 
 # INITIALIZATION
 
+has logo     => ( is => 'rw', default => 'webauthn.png' );
 has prefix   => ( is => 'rw', default => 'webauthn' );
 has template => ( is => 'ro', default => 'webauthn2fregister' );
 has welcome  => ( is => 'ro', default => 'webauthn2fWelcome' );
-has logo     => ( is => 'rw', default => 'webauthn.png' );
 has ott => (
     is      => 'ro',
     lazy    => 1,
@@ -80,15 +80,10 @@ sub _generate_user_handle {
 sub _registrationchallenge {
     my ( $self, $req, $user ) = @_;
 
-    my @alldevices = $self->find2fDevicesByType( $req, $req->userData );
-    my $size       = @alldevices;
-    my $maxSize    = $self->conf->{max2FDevices};
-    $self->logger->debug("Registered 2F Device(s): $size / $maxSize");
-    if ( $size >= $maxSize ) {
-        $self->userLogger->warn("Max number of 2F devices is reached");
-        return $self->p->sendError( $req, 'maxNumberof2FDevicesReached', 400 );
-    }
+    $self->checkCsrf($req)
+        or return $self->p->sendError( $req, 'csrfError', 400 );
 
+    my @alldevices       = $self->find2fDevicesByType( $req, $req->userData );
     my $challenge_base64 = encode_base64url( Crypt::URandom::urandom(32) );
 
     # Challenge is persisted on the server
@@ -101,8 +96,7 @@ sub _registrationchallenge {
 
     my $displayName      = $req->userData->{ $self->displayname_attr } || $user;
     my $userVerification = $self->conf->{webauthn2fUserVerification};
-
-    my $request = {
+    my $request          = {
         rp => {
             name => $self->rpName,
         },
@@ -129,108 +123,105 @@ sub _registrationchallenge {
 sub _registration {
     my ( $self, $req, $user ) = @_;
 
+    $self->checkCsrf($req)
+        or return $self->p->sendError( $req, 'csrfError', 400 );
+
     # Recover creation parameters, including challenge
     my $state_id = $req->param('state_id');
     unless ($state_id) {
-        $self->logger->error("Could not find state ID in WebAuthn response");
-        return $self->p->sendError( $req, 'Invalid response', 400 );
+        $self->logger->error(
+            $self->prefix . "2f: could not find state ID in response" );
+        return $self->p->sendError( $req, 'webAuthnRegisterFailed', 400 );
     }
     my $state_data;
     unless ( $state_data = $self->ott->getToken($state_id) ) {
-        $self->logger->error(
-            "Expired or invalid state ID in WebAuthn response: $state_id");
+        $self->logger->error( $self->prefix
+              . "2f: expired or invalid state ID in response: $state_id" );
         return $self->p->sendError( $req, 'PE82', 400 );
     }
     my $registration_options = ( $state_data->{registration_options} );
-    return $self->p->sendError( $req,
-        'Registration options missing from state data', 400 )
-      unless ($registration_options);
+    unless ($registration_options) {
+        $self->logger->error( $self->prefix
+              . '2f: registration options missing from state data' );
+        return $self->p->sendError( $req, 'webAuthnRegisterFailed', 400 );
+    }
 
     # Data required for WebAuthn verification
     my $credential_json = $req->param('credential');
-    $self->logger->debug("Get registered credential data $credential_json");
+    $self->logger->debug(
+        $self->prefix . "2f: get registered credential data $credential_json" );
 
-    return $self->p->sendError( $req, 'Missing credential parameter', 400 )
-      unless ($credential_json);
+    unless ($credential_json) {
+        $self->logger->error(
+            $self->prefix . '2f: missing credential parameter' );
+        return $self->p->sendError( $req, 'webAuthnRegisterFailed', 400 );
+    }
 
     my $validation = eval {
         $self->validateCredential( $req, $registration_options,
             $credential_json );
     };
     if ($@) {
-        $self->logger->error("Credential validation error: $@");
+        $self->logger->error(
+            $self->prefix . "2f: Credential validation error: $@" );
         return $self->p->sendError( $req, "webAuthnRegisterFailed", 400 );
     }
 
     my $credential_id     = $validation->{credential_id};
     my $credential_pubkey = $validation->{credential_pubkey};
     my $signature_count   = $validation->{signature_count};
+    $self->logger->debug( $self->prefix
+          . "2f: registering new credential: \n"
+          . "ID: $credential_id\n"
+          . "Public key: $credential_pubkey\n"
+          . "Signature count: $signature_count" );
 
-    $self->logger->debug( "Registering new credential : \n" . "ID: "
-          . $credential_id . "\n"
-          . "Public key "
-          . $credential_pubkey . "\n"
-          . "Signature count "
-          . $signature_count
-          . "\n" );
+    return $self->p->sendError( $req, 'webauthnAlreadyRegistered', 400 )
+      if $self->find2fDevicesByKey( $req, $req->userData, $self->type,
+        "_credentialId", $credential_id );
 
-    if (
-        $self->find2fDevicesByKey(
-            $req, $req->userData, $self->type,
-            "_credentialId", $credential_id
-        )
-      )
-    {
-        return $self->p->sendError( $req, 'webauthnAlreadyRegistered', 400 );
-    }
-
-    my $keyName = $req->param('keyName');
-    my $epoch   = time();
-
-    # Set default name if empty, check characters and truncate name if too long
-    $keyName ||= $epoch;
-    unless ( $keyName =~ /^[\w]+$/ ) {
-        $self->userLogger->error('WebAuthn name with bad character(s)');
-        return $self->p->sendError( $req, 'badName', 200 );
-    }
-    $keyName = substr( $keyName, 0, $self->conf->{max2FDevicesNameLength} );
-    $self->logger->debug("Key name: $keyName");
+    my $keyName =
+      $self->checkNameSfa( $req, $self->type, $req->param('keyName') );
+    return $self->p->sendError( $req, 'badName', 200 ) unless $keyName;
 
     if (
         $self->add2fDevice(
             $req,
             $req->userData,
             {
-                type                 => $self->type,
-                name                 => $keyName,
                 _credentialId        => $credential_id,
                 _credentialPublicKey => $credential_pubkey,
                 _signCount           => $signature_count,
-                epoch                => $epoch
+                type                 => $self->type,
+                name                 => $keyName,
+                epoch                => time()
             }
         )
       )
     {
         $self->userLogger->notice(
-            "Webauthn key registration of $keyName succeeds for $user");
-
+            $self->prefix . "2f: registration of $keyName succeeds for $user" );
         return $self->p->sendJSONresponse( $req, { result => 1 } );
     }
     else {
-        return $self->p->sendError( $req, 'Server Error', 500 );
+        $self->logger->error( $self->prefix . '2f: unable to add device' );
+        return $self->p->sendError( $req, 'serverError' );
     }
-
 }
 
 sub _verificationchallenge {
     my ( $self, $req, $user ) = @_;
 
-    $self->logger->debug('Verification challenge req');
+    $self->checkCsrf($req)
+        or return $self->p->sendError( $req, 'csrfError', 400 );
+
+    $self->logger->debug( $self->prefix . '2f: verification challenge req' );
 
     my $request = $self->generateChallenge( $req, $req->userData );
 
     unless ($request) {
-        return $self->p->sendError( $req, "No registered devices", 400 );
+        $self->logger->error( $self->prefix . '2f: no registered device' );
+        return $self->p->sendError( $req, 'webAuthnNoDevice', 500 );
     }
 
     # Request is persisted on the server
@@ -239,7 +230,8 @@ sub _verificationchallenge {
         }
     );
 
-    $self->logger->debug( "Authentication parameters: " . to_json($request) );
+    $self->logger->debug(
+        $self->prefix . "2f: authentication parameters: " . to_json($request) );
     return $self->p->sendJSONresponse( $req,
         { request => $request, state_id => $token } );
 }
@@ -247,34 +239,40 @@ sub _verificationchallenge {
 sub _verification {
     my ( $self, $req, $user ) = @_;
 
+    $self->checkCsrf($req)
+        or return $self->p->sendError( $req, 'csrfError', 400 );
+
     my $credential_json = $req->param('credential');
 
-    return $self->p->sendError( $req, 'Missing credential parameter', 400 )
-      unless ($credential_json);
+    unless ($credential_json) {
+        $self->logger->error(
+            $self->prefix . '2f: missing credential parameter' );
+        return $self->p->sendError( $req, 'webAuthnFailed', 400 );
+    }
 
     my $state_id = $req->param('state_id');
     unless ($state_id) {
-        $self->logger->error(
-            "Could not find state ID in WebAuthn response: $credential_json");
-        return $self->p->sendError( $req, 'Invalid response', 400 );
+        $self->logger->error( $self->prefix
+              . "2f: could not find state ID in response ($credential_json)" );
+        return $self->p->sendError( $req, 'webAuthnFailed', 400 );
     }
 
     # Recover challenge
     my $state_data;
     unless ( $state_data = $self->ott->getToken($state_id) ) {
-        $self->logger->error(
-            "Expired or invalid state ID in WebAuthn response: $state_id");
+        $self->logger->error( $self->prefix
+              . "2f: expired or invalid state ID in response ($state_id)" );
         return $self->p->sendError( $req, 'PE82', 400 );
     }
 
     my $signature_options = ( $state_data->{authentication_options} );
-
     my $validation_result = eval {
         $self->validateAssertion( $req, $req->userData, $signature_options,
             $credential_json );
     };
     if ($@) {
-        $self->logger->error("Webauthn validation error for $user: $@");
+        $self->logger->error(
+            $self->prefix . "2f: validation error for $user: $@" );
         return $self->p->sendJSONresponse( $req, { result => 0 } );
     }
 
@@ -289,20 +287,23 @@ sub _verification {
 sub _delete {
     my ( $self, $req, $user ) = @_;
 
+    $self->checkCsrf($req)
+        or return $self->p->sendError( $req, 'csrfError', 400 );
+
     # Check if unregistration is allowed
-    return $self->p->sendError( $req, 'notAuthorized', 200 )
-      unless $self->conf->{webauthn2fUserCanRemoveKey};
+    return $self->p->sendError( $req, 'notAuthorized', 400 )
+      unless $self->userCanRemove;
 
-    my $epoch = $req->param('epoch')
-      or
-      return $self->p->sendError( $req, '"epoch" parameter is missing', 400 );
-
-    if ( $self->del2fDevice( $req, $req->userData, $self->type, $epoch ) ) {
-        return $self->p->sendJSONresponse( $req, { result => 1 } );
-    }
-    else {
+    my $epoch = $req->param('epoch');
+    unless ($epoch) {
+        $self->logger->error(
+            $self->prefix . '2f: "epoch" parameter is missing' );
         return $self->p->sendError( $req, '2FDeviceNotFound', 400 );
     }
+
+    return $self->del2fDevice( $req, $req->userData, $self->type, $epoch )
+      ? $self->p->sendJSONresponse( $req, { result => 1 } )
+      : $self->p->sendError( $req, '2FDeviceNotFound', 400 );
 }
 
 # Main method
@@ -310,13 +311,11 @@ sub run {
     my ( $self, $req, $action ) = @_;
     my $user = $req->userData->{ $self->conf->{whatToTrace} };
 
-    return $self->p->sendError( $req,
-        'No ' . $self->conf->{whatToTrace} . ' found in user data', 500 )
-      unless $user;
-
-    # Check if U2F key can be updated
-    my $msg = $self->canUpdateSfa( $req, $action );
-    return $self->p->sendError( $req, $msg, 400 ) if $msg;
+    unless ($user) {
+        $self->logger->error(
+            'No ' . $self->conf->{whatToTrace} . ' found in user data' );
+        return $self->p->sendError( $req, 'serverError', 500 );
+    }
 
     if ( $action eq 'registrationchallenge' ) {
         return $self->_registrationchallenge( $req, $user );
@@ -339,10 +338,9 @@ sub run {
     }
 
     else {
-        $self->logger->error("Unknown WebAuthn action -> $action");
+        $self->logger->error( $self->prefix . "2f: unknown action -> $action" );
         return $self->p->sendError( $req, 'unknownAction', 400 );
     }
-
 }
 
 1;
