@@ -17,8 +17,9 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_UNAUTHORIZEDPARTNER
   PE_CAS_SERVICE_NOT_ALLOWED
 );
+use URI;
 
-our $VERSION = '2.0.15';
+our $VERSION = '2.16.3';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::CAS';
@@ -93,7 +94,8 @@ sub storeEnvAndCheckGateway {
         if ( $self->_gatewayAllowedRedirect( $req, $service ) ) {
             $self->logger->debug(
                 "Gateway mode requested, redirect without authentication");
-            $req->response( [ 302, [ Location => $service ], [] ] );
+            $req->response(
+                [ 302, [ Location => URI->new($service)->as_string ], [] ] );
             for my $s ( $self->ipath, $self->ipath . 'Path' ) {
                 $self->logger->debug("Removing $s from pdata")
                   if delete $req->pdata->{$s};
