@@ -9,6 +9,8 @@ use DBI;
 use MIME::Base64;
 use strict;
 use Mouse;
+use Crypt::URandom;
+use MIME::Base64 qw/encode_base64url/;
 
 extends 'Lemonldap::NG::Common::Module';
 
@@ -95,10 +97,9 @@ sub hash_password {
         $self->logger->debug( "Using " . uc($hash) . " to hash password" );
         return uc($hash) . "($password)";
     }
-    else {
-        $self->logger->debug("No password hash, using clear text for password");
-        return $password;
-    }
+
+    $self->logger->debug("No password hash, using clear text for password");
+    return $password;
 }
 
 # Return hashed password for use in SQL SELECT statement
@@ -208,6 +209,68 @@ sub hash_password_from_database {
     # Return encode_base64(SQL_METHOD(password + salt) + salt)
 }
 
+## @method protected Lemonldap::NG::Portal::_DBI hash_unix_password_from_database
+##  (ref dbh, string dbmethod, string dbsalt, string password)
+# Hash the given password calling the dbmethod function in database
+# @param dbh database handler
+# @param dbunixsalth the unix salt including prefix and converted to hex (optional hash)
+# @param password the password to hash
+# @return unix password hash
+sub hash_unix_password_from_database {
+
+    # Remark: database function must get hexadecimal input
+    # and send back hexadecimal output
+    my $self        = shift;
+    my $dbh         = shift;
+    my $dbunixsalth = shift;
+    my $password    = shift;
+
+    # convert password to hexa
+    my $passwordh = unpack "H*", $password;
+
+    # If an unix hashing algorithm is specifid:
+    # We need to give a salt and algorithm as second parameter.
+    # A unix salted password string starts with algorithm,
+    # then salt and ends with the hash.
+    # eg. $6$ telling crypt that we want sha512...
+    # $2a$ and $2y$ not working in latest mariadb. If wanted
+    # there is additional research nessesary!
+    my @rows = ();
+    eval {
+        my $sth =
+          $dbh->prepare("SELECT unixcrypth('$passwordh', '$dbunixsalth')");
+        $sth->execute();
+        @rows = $sth->fetchrow_array();
+    };
+    if ($@) {
+        $self->logger->error(
+            "DBI error while hashing with unixcrypth hash function: $@");
+        $self->userLogger->warn("Unable to check password");
+        return "";
+    }
+
+    if ( @rows == 1 ) {
+        $self->logger->debug(
+"Successfully hashed password with unixcrypth hash function in database"
+        );
+
+        # convert salt to binary
+        # my $dbsaltb = pack 'H*', $dbsalt;
+
+        # convert result to binary
+        my $res = pack 'H*', $rows[0];
+
+        # res string contains all information, methode, salt and hash
+        return $res;
+    }
+    else {
+        $self->userLogger->warn("Unable to check password with unixcrypth");
+        return "";
+    }
+
+    # Return encode_base64(unixcrypth($n$salt$hash))
+}
+
 ## @method protected Lemonldap::NG::Portal::_DBI get_salt(string dbhash)
 # Return salt from salted hash password
 # @param dbhash hash password
@@ -245,6 +308,18 @@ sub gen_salt {
     return $dbsalt;
 }
 
+## @method protected Lemonldap::NG::Portal::_DBI gen_salt_text()
+# Generate 16 bytes of text random salt
+# @return generated salt text
+sub gen_salt_text {
+    my $self   = shift;
+    my $dbsalt = encode_base64url( Crypt::URandom::urandom(16) );
+    $dbsalt =~ s/-/\./g;
+    $dbsalt =~ s/_/\//g;
+
+    return substr( $dbsalt, 0, 16 );
+}
+
 ## @method protected Lemonldap::NG::Portal::_DBI dynamic_hash_password(ref dbh,
 ##  string user, string password, string table, string loginCol, string passwordCol)
 # Return hashed password for use in SQL statement
@@ -285,6 +360,33 @@ sub dynamic_hash_password {
     $dbscheme =~ s/^\{([^}]+)\}.*/$1/;
     $dbscheme = "" if $dbscheme eq $dbhash;
 
+    # check for unix style passwords
+    if ( $dbscheme eq "" and $dbhash =~ /^\$(1|5|6)\$/i ) {
+        $dbscheme = $1;
+
+        #check if unix hash scheme is valid
+        if ( not grep( /^unixcrypt$dbscheme$/, @validSaltedSchemes ) ) {
+            $self->logger->error(
+                "No valid unix hash scheme: unixcrypt$dbscheme for user $user");
+            $self->userLogger->warn("Unable to check password for $user");
+            return "";
+        }
+
+        $self->logger->debug(
+            "Using unixcrypt$dbscheme to hash salted password");
+
+        # Hex the salt
+        my $dbhash = unpack "H*", $dbhash;
+
+        # Let the db generate a unix hash
+        # The dbhash contains the hash and the encrypt function will
+        # ignore the rest (algorithm, hash)
+        $hash =
+          $self->hash_unix_password_from_database( $dbh, $dbhash, $password );
+        return "'$hash'";
+
+    }
+
     # no hash scheme => assume clear text
     if ( $dbscheme eq "" ) {
         $self->logger->info("Password has no hash scheme");
@@ -293,7 +395,7 @@ sub dynamic_hash_password {
     }
 
     # salted hash scheme
-    elsif ( grep( /^$dbscheme$/, @validSaltedSchemes ) ) {
+    if ( grep( /^$dbscheme$/, @validSaltedSchemes ) ) {
         $self->logger->info(
             "Valid salted hash scheme: $dbscheme found for user $user");
 
@@ -369,6 +471,21 @@ sub dynamic_hash_new_password {
 
     }
 
+    # Check for unix password string
+    elsif ( $dbscheme =~ /^unixcrypt(1|5|6)$/i ) {
+        $self->logger->info("Selected salted hash scheme: $dbscheme");
+
+        $dbscheme = $1;
+        $dbsalt   = gen_salt_text();
+        $dbsalt   = unpack "H*", "\$$dbscheme\$$dbsalt\$";
+
+        $hash =
+          $self->hash_unix_password_from_database( $dbh, $dbsalt, $password );
+
+        return "'$hash'";
+
+    }
+
     # salted hash scheme
     elsif ( grep( /^$dbscheme$/, @validSaltedSchemes ) ) {
         $self->logger->info("Selected salted hash scheme: $dbscheme");
