@@ -24,10 +24,11 @@ use URI;
 use URI::QueryParam;
 use Mouse;
 use Crypt::URandom;
+use URI;
 
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_REDIRECT PE_ERROR);
 
-our $VERSION = '2.0.15';
+our $VERSION = '2.16.3';
 
 # OpenID Connect standard claims
 use constant PROFILE => [
@@ -2288,7 +2289,7 @@ sub buildLogoutResponse {
         $response_url .= build_urlencoded( state => $state );
     }
 
-    return $response_url;
+    return URI->new($response_url)->as_string;
 }
 
 # Create session_state parameter
@@ -2353,6 +2354,11 @@ sub addRouteFromConf {
 sub validatePKCEChallenge {
     my ( $self, $code_verifier, $code_challenge, $code_challenge_method ) = @_;
 
+    unless ($code_challenge) {
+        $self->logger->debug("PKCE was not requested by the RP");
+        return 1;
+    }
+
     unless ($code_verifier) {
         $self->logger->error("PKCE required but no code verifier provided");
         return 0;
