@@ -1,6 +1,6 @@
 package Lemonldap::NG::Portal::Main::Process;
 
-our $VERSION = '2.0.15';
+our $VERSION = '2.16.3';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -8,6 +8,7 @@ use strict;
 use MIME::Base64;
 use POSIX qw(strftime);
 use Lemonldap::NG::Portal::Main::Constants qw(portalConsts);
+use URI;
 
 # Main method
 # -----------
@@ -51,6 +52,8 @@ sub processHook {
     my $err = PE_OK;
     for my $sub ( @{ $self->hook->{$hookName} } ) {
         if ( ref $sub eq 'CODE' ) {
+
+            # If return code is not PE_OK (0), stop processing
             last if ( $err = $sub->( $req, @args ) );
         }
         else {
@@ -58,8 +61,14 @@ sub processHook {
         }
     }
     if ( $err != PE_OK ) {
-        $self->logger->warn(
-            "Hook $hookName returned " . portalConsts->{$err} );
+        my $msg = "Hook $hookName returned " . portalConsts->{$err};
+        if ( $err > 0 ) {
+            $self->logger->warn($msg);
+        }
+        else {
+            # <0 error codes are normal in some use cases
+            $self->logger->debug($msg);
+        }
     }
     return $err;
 }
@@ -135,6 +144,7 @@ sub controlUrl {
                 $req->{urldc} =~ s/[\r\n]//sg;
             }
         }
+        $req->{urldc} = URI->new( $req->{urldc} )->as_string;
 
         # For logout request, test if Referer comes from an authorized site
         my $tmp = (
@@ -161,7 +171,15 @@ sub controlUrl {
             delete $req->{urldc};
             return PE_BADURL;
         }
-        my ( $proto, $vhost, $appuri ) = ( $2, $3, $5 );
+        my ( $proto, $vhost, $appuri );
+        if ($tmp) {
+            my $u = URI->new($tmp);
+            if ( $u->scheme =~ /^https?$/ ) {
+                $proto  = $u->scheme;
+                $vhost  = $u->host if $u->can("host");
+                $appuri = $u->path_query;
+            }
+        }
 
         # Try to resolve alias
         my $originalVhost = $self->HANDLER->resolveAlias($vhost);
@@ -179,11 +197,11 @@ sub controlUrl {
             and !$self->isTrustedUrl($vhost) )
         {
             $self->userLogger->error(
-                    "URL contains a non protected host (param: "
+                    "URL contains an unprotected host (param: "
                   . ( $req->param('logout') ? 'HTTP Referer' : 'urldc' )
                   . " | value: $tmp | alias: $vhost)" );
             delete $req->{urldc};
-            return PE_BADURL;
+            return PE_UNPROTECTEDURL;
         }
 
         $req->env->{urldc} = $req->{urldc};
@@ -303,16 +321,16 @@ sub deleteSession {
     $req->userData( {} );
 
     # Redirect or Post if asked by authLogout
-    if ( $req->urldc and $req->urldc ne $self->conf->{portal} ) {
-        $req->steps( [] );
-        return PE_REDIRECT;
-    }
-
     if ( $req->postUrl ) {
         $req->steps( ['autoPost'] );
         return PE_OK;
     }
 
+    if ( $req->urldc and $req->urldc ne $self->conf->{portal} ) {
+        $req->steps( [] );
+        return PE_REDIRECT;
+    }
+
     # If logout redirects to another URL, just remove next steps for the
     # request so autoRedirect will be called
     if ( $req->{urldc} and $req->{urldc} ne $self->conf->{portal} ) {
@@ -480,22 +498,19 @@ sub setGroups {
 sub setPersistentSessionInfo {
     my ( $self, $req ) = @_;
 
-    # Do not restore infos if session already opened
-    unless ( $req->id ) {
-        my $key = $req->{sessionInfo}->{ $self->conf->{whatToTrace} };
+    my $key = $req->{sessionInfo}->{ $self->conf->{whatToTrace} };
 
-        return PE_OK unless ( $key and length($key) );
+    return PE_OK unless ( $key and length($key) );
 
-        my $persistentSession = $self->getPersistentSession($key);
-        if ($persistentSession) {
-            $self->logger->debug("Persistent session found for $key");
-            foreach my $k ( keys %{ $persistentSession->data } ) {
+    my $persistentSession = $self->getPersistentSession($key);
+    if ($persistentSession) {
+        $self->logger->debug("Persistent session found for $key");
+        foreach my $k ( keys %{ $persistentSession->data } ) {
 
-                # Do not restore some parameters
-                next if $k =~ /^_(?:utime|session_(?:u?id|kind))$/;
-                $self->logger->debug("Restore persistent parameter $k");
-                $req->{sessionInfo}->{$k} = $persistentSession->data->{$k};
-            }
+            # Do not restore some parameters
+            next if $k =~ /^_(?:utime|session_(?:u?id|kind))$/;
+            $self->logger->debug("Restore persistent parameter $k");
+            $req->{sessionInfo}->{$k} = $persistentSession->data->{$k};
         }
     }
     return PE_OK;
