@@ -32,7 +32,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_PP_INSUFFICIENT_PASSWORD_QUALITY
 );
 
-our $VERSION = '2.0.16';
+our $VERSION = '2.16.1';
 
 extends qw(
   Lemonldap::NG::Portal::Lib::SMTP
@@ -42,6 +42,18 @@ extends qw(
 
 # PROPERTIES
 
+# Reset URL to set in mail
+has resetUrl => (
+    is      => 'ro',
+    lazy    => 1,
+    default => sub {
+        my $self = $_[0];
+        my $p    = $self->conf->{portal};
+        $p =~ s#/*$##;
+        return $self->conf->{mailUrl} ? $self->conf->{mailUrl} : "$p/resetpwd";
+    }
+);
+
 # Mail timeout token generator
 # Form timout token generator (used even if requireToken is not set)
 has ott => (
@@ -50,7 +62,8 @@ has ott => (
     default => sub {
         my $ott =
           $_[0]->{p}->loadModule('Lemonldap::NG::Portal::Lib::OneTimeToken');
-        $ott->timeout( $_[0]->conf->{formTimeout} );
+        $ott->cache(0);
+        $ott->timeout( $_[0]->conf->{formTimeout} || $_[0]->conf->{timeout} );
         return $ott;
     }
 );
@@ -308,7 +321,7 @@ sub _reset {
         my $req_url = $req->data->{_url};
         my $skin    = $self->p->getSkin($req);
         my $url =
-          $self->conf->{mailUrl} . '?'
+          $self->resetUrl . '?'
           . build_urlencoded(
             mail_token => $req->{id},
             skin       => $skin,
@@ -448,7 +461,7 @@ sub changePwd {
             $pwdRegEx = "[A-Z]{$uppers}[a-z]{$lowers}\\d{$digits}";
             $pwdRegEx .=
               $self->conf->{passwordPolicySpecialChar} eq '__ALL__'
-              ? qq[\W{$chars}]
+              ? '\W{$chars}'
               : "[$self->{conf}->{passwordPolicySpecialChar}]{$chars}";
             $self->logger->debug("Generated password RegEx: $pwdRegEx");
         }
