@@ -3,7 +3,7 @@ package t::OidcHookPlugin;
 use Mouse;
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
-use Lemonldap::NG::Portal::Main::Constants qw(PE_OK);
+use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_SENDRESPONSE);
 use Data::Dumper;
 use Test::More;
 
@@ -21,6 +21,7 @@ use constant hook => {
     oidcGotIDToken                    => 'modifyIDToken',
     oidcGotOnlineRefresh              => 'refreshHook',
     oidcGotOfflineRefresh             => 'refreshHook',
+    oidcGotTokenExchange              => 'tokenExchange',
 };
 
 sub addClaimToIDToken {
@@ -107,4 +108,13 @@ sub refreshHook {
     return PE_OK;
 }
 
+sub tokenExchange {
+    my ( $self, $req, $rp ) = @_;
+    if ( $req->param("testtokenexchange") ) {
+        $req->response( $self->p->sendJSONresponse( $req, { result => 1 } ) );
+        return PE_SENDRESPONSE;
+    }
+    return PE_OK;
+}
+
 1;
