@@ -73,7 +73,8 @@ sub login {
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     my ( $op, $uid ) = @_;
     my $res;
-    my $query = buildForm( {
+    my $query = buildForm(
+        {
             user     => $uid,
             password => $uid,
         }
@@ -110,7 +111,8 @@ sub codeAuthorize {
 
 sub tokenExchange {
     my ( $op, $clientid, %params ) = @_;
-    my $query = buildForm( {
+    my $query = buildForm(
+        {
             grant_type => 'urn:ietf:params:oauth:grant-type:token-exchange',
             %params
         }
@@ -131,7 +133,8 @@ sub tokenExchange {
 
 sub codeGrant {
     my ( $op, $clientid, $code, $redirect_uri ) = @_;
-    my $query = buildForm( {
+    my $query = buildForm(
+        {
             grant_type   => "authorization_code",
             code         => $code,
             redirect_uri => $redirect_uri,
@@ -168,7 +171,8 @@ sub getUserinfo {
 sub refreshGrant {
     my ( $op, $client_id, $refresh_token ) = @_;
 
-    $query = buildForm( {
+    $query = buildForm(
+        {
             grant_type    => 'refresh_token',
             refresh_token => $refresh_token,
         }
@@ -189,7 +193,8 @@ sub refreshGrant {
 
 sub introspect {
     my ( $op, $client_id, $token ) = @_;
-    my $query = buildForm( {
+    my $query = buildForm(
+        {
             client_id     => $client_id,
             client_secret => $client_id,
             token         => $token,
